@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -23,14 +23,19 @@
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 from django.contrib.auth import get_user_model
-from rest_framework.generics import ListAPIView
+from rest_framework import response as http, status
+from rest_framework.generics import (ListAPIView, ListCreateAPIView,
+    RetrieveAPIView)
 
 from .organizations import OrganizationQuerysetMixin
-from .serializers import OrganizationSerializer
-from .. import filters
-from ..mixins import (OrganizationSmartListMixin, UserSmartListMixin)
+from .serializers import (OrganizationSerializer, OrganizationCreateSerializer,
+    OrganizationDetailSerializer)
+from .. import filters, settings
+from ..docs import OpenAPIResponse, swagger_auto_schema
+from ..mixins import (OrganizationCreateMixin, OrganizationSmartListMixin,
+    OrganizationMixin, OrganizationDecorateMixin, UserSmartListMixin)
 from ..pagination import TypeaheadPagination
-from ..utils import get_user_serializer
+from ..utils import get_organization_model, get_user_serializer
 
 
 #pylint: disable=no-init
@@ -194,7 +199,8 @@ class AccountsTypeaheadAPIView(OrganizationSmartListMixin,
 
 
 class ProfilesTypeaheadAPIView(OrganizationSmartListMixin,
-                            OrganizationQuerysetMixin, ListAPIView):
+                               OrganizationQuerysetMixin,
+                               OrganizationCreateMixin, ListCreateAPIView):
     """
     Searches profiles
 
@@ -241,11 +247,119 @@ class ProfilesTypeaheadAPIView(OrganizationSmartListMixin,
     serializer_class = OrganizationSerializer
     pagination_class = TypeaheadPagination
 
+    def get_serializer_class(self):
+        if self.request.method.lower() == 'post':
+            return OrganizationCreateSerializer
+        return super(ProfilesTypeaheadAPIView, self).get_serializer_class()
+
+    @swagger_auto_schema(responses={
+      201: OpenAPIResponse("Create successful", OrganizationDetailSerializer)})
+    def post(self, request, *args, **kwargs):
+        """
+        Creates a shadow profile
+
+        **Examples**
+
+        .. code-block:: http
+
+            POST /api/accounts/profiles/ HTTP/1.1
+
+        .. code-block:: json
+
+            {
+              "email": "xia@locahost.localdomain",
+              "full_name": "Xia Lee",
+              "type": "personal"
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+              "slug": "xia",
+              "email": "xia@locahost.localdomain",
+              "full_name": "Xia Lee",
+              "printable_name": "Xia Lee",
+              "type": "personal",
+              "credentials": true,
+              "default_timezone": "America/Los_Angeles",
+              "phone": "",
+              "street_address": "",
+              "locality": "",
+              "region": "",
+              "postal_code": "",
+              "country": "US",
+              "is_bulk_buyer": false,
+              "extra": null
+            }
+
+        """
+        return self.create(request, *args, **kwargs)
+
     def paginate_queryset(self, queryset):
         page = super(ProfilesTypeaheadAPIView, self).paginate_queryset(queryset)
         page = self.decorate_personal(page)
         return page
 
+    def create(self, request, *args, **kwargs):
+        #pylint:disable=unused-argument
+        serializer = OrganizationCreateSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+
+        # creates profile
+        organization = self.create_organization(serializer.validated_data)
+        self.decorate_personal(organization)
+
+        # returns created profile
+        serializer = OrganizationDetailSerializer(
+            instance=organization,
+            context=self.get_serializer_context())
+        headers = self.get_success_headers(serializer.data)
+        return http.Response(serializer.data,
+            status=status.HTTP_201_CREATED, headers=headers)
+
+
+
+class ProfileAPIView(OrganizationMixin, OrganizationDecorateMixin,
+                     RetrieveAPIView):
+    """
+    Retrieves a billing profile
+
+    The API is typically used within an HTML
+    `contact information page </docs/themes/#dashboard_profile>`_
+    as present in the default theme.
+
+    **Tags**: profile, subscriber, profilemodel
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/accounts/profiles/xia/ HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "slug": "xia",
+            "printable_name": "Xia Lee",
+            "type": "organization",
+            "picture": null
+        }
+    """
+    lookup_field = 'slug'
+    lookup_url_kwarg = settings.PROFILE_URL_KWARG
+    queryset = get_organization_model().objects.all()
+    serializer_class = OrganizationSerializer
+    user_model = get_user_model()
+
+    def get_object(self):
+        obj = super(ProfileAPIView, self).get_object()
+        self.decorate_personal(obj)
+        return obj
+
 
 class UserQuerysetMixin(object):
     """
