@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,7 +24,6 @@
 
 #pylint:disable=useless-super-delegation
 
-from django.utils.translation import ugettext_lazy as _
 from rest_framework import status
 from rest_framework.exceptions import ValidationError
 from rest_framework.generics import (RetrieveAPIView,
@@ -32,6 +31,7 @@ from rest_framework.generics import (RetrieveAPIView,
 from rest_framework.response import Response
 
 from ..backends import ProcessorError
+from ..compat import gettext_lazy as _
 from ..docs import OpenAPIResponse, swagger_auto_schema, Parameter, IN_PATH
 from ..mixins import OrganizationMixin
 from ..models import get_broker
@@ -203,10 +203,11 @@ class PaymentMethodDetailAPIView(OrganizationMixin,
         #pylint:disable=unused-argument
         resp_data = self.organization.retrieve_card()
         if request.query_params.get('update', False):
+            broker = get_broker()
             resp_data.update({
                 'processor':
-                self.organization.processor_backend.get_payment_context(
-                    get_broker(),
+                broker.processor_backend.get_payment_context(
+                    broker,
                     self.organization.processor_card_key,
                     subscriber_email=self.organization.email,
                     subscriber_slug=self.organization.slug)
