@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,6 @@ from __future__ import unicode_literals
 import csv, logging
 
 from django.contrib import messages
-from django.utils.translation import ugettext_lazy as _
 from rest_framework.generics import (CreateAPIView,
     GenericAPIView, RetrieveAPIView)
 from rest_framework.mixins import CreateModelMixin
@@ -38,13 +37,15 @@ from rest_framework.response import Response
 from rest_framework import status
 
 from ..backends import ProcessorError
-from ..compat import is_authenticated, StringIO
+from ..compat import gettext_lazy as _, is_authenticated, StringIO
 from ..docs import swagger_auto_schema, OpenAPIResponse
 from ..mixins import BalanceAndCartMixin, CartMixin, InvoicablesMixin
 from ..models import CartItem
-from .serializers import (CartItemCreateSerializer, CartItemUploadSerializer,
-    ChargeSerializer, CheckoutSerializer, OrganizationCartSerializer,
-    RedeemCouponSerializer, ValidationErrorSerializer)
+from ..utils import datetime_or_now
+from .serializers import (CartItemSerializer, CartItemCreateSerializer,
+    CartItemUploadSerializer, ChargeSerializer, CheckoutSerializer,
+    OrganizationCartSerializer, RedeemCouponSerializer,
+    ValidationErrorSerializer)
 
 #pylint: disable=no-init
 LOGGER = logging.getLogger(__name__)
@@ -52,7 +53,7 @@ LOGGER = logging.getLogger(__name__)
 
 class CartItemAPIView(CartMixin, CreateAPIView):
     """
-    Adds an item to the request user cart
+    Adds an item to the user cart
 
     Adds a plan into the cart of the user identified through the HTTP request.
 
@@ -95,8 +96,17 @@ class CartItemAPIView(CartMixin, CreateAPIView):
     .. code-block:: json
 
         {
-            "plan": "premium",
-            "option": 1
+            "plan": {
+              "slug": "premium",
+              "title": "Premium"
+            },
+            "option": 1,
+            "user": {
+              "username": "xia",
+              "slug": "xia",
+              "full_name": "Xia Lee",
+              "email": "xia@localhost.localdomain"
+            }
         }
 
     """
@@ -109,6 +119,9 @@ class CartItemAPIView(CartMixin, CreateAPIView):
     # and csrf, unfortunately it prevents authenticated users to add into
     # their db cart, instead put their choices into the unauth session.
     # authentication_classes = []
+    @swagger_auto_schema(responses={
+      200: OpenAPIResponse("updated", CartItemSerializer),
+      201: OpenAPIResponse("created", CartItemSerializer)})
     def post(self, request, *args, **kwargs):
         items = None
         serializer = self.get_serializer(data=request.data)
@@ -123,17 +136,18 @@ class CartItemAPIView(CartMixin, CreateAPIView):
                 serializer.errors, status=status.HTTP_400_BAD_REQUEST)
 
         cart_items = []
+        at_time = datetime_or_now()
         status_code = status.HTTP_200_OK
+        serializer = CartItemSerializer()
         for item in items:
-            cart_item, created = self.insert_item(request, **item)
+            cart_item, created = self.insert_item(
+                request, at_time=at_time, **item)
             if created:
                 status_code = status.HTTP_201_CREATED
             # insert_item will either return a dict or a CartItem instance
             # (which cannot be directly serialized).
             if isinstance(cart_item, CartItem):
                 cart_item = serializer.to_representation(cart_item)
-            if cart_item.get('sync_on'):
-                cart_item.update({'detail': _("User was added.")})
             cart_items += [cart_item]
         if len(items) > 1:
             headers = self.get_success_headers(cart_items)
@@ -169,7 +183,7 @@ class CartItemAPIView(CartMixin, CreateAPIView):
 
     def delete(self, request, *args, **kwargs):
         """
-        Removes an item from the request user cart
+        Removes an item from the user cart
 
         Removes an item from the ``request.user`` cart.
 
@@ -196,7 +210,7 @@ class CartItemAPIView(CartMixin, CreateAPIView):
 
 class CartItemUploadAPIView(CartMixin, GenericAPIView):
     """
-    Uploads multiple items into a cart
+    Uploads multiple items into a user cart
 
     Add a ``Plan`` into the subscription cart of multiple users as per the
     content of an uploaded file.
@@ -231,16 +245,28 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
         {
             "created": [
                 {
-                    "first_name": "Joe",
-                    "last_name": "Smith",
-                    "email": "joesmith@example.com",
-                    "plan": "basic"
+                    "plan": {
+                      "slug": "basic",
+                      "title": "Basic"
+                    },
+                    "user": {
+                        "username": "joe",
+                        "slug": "joe",
+                        "full_name": "Joe Smith",
+                        "email": "joesmith@example.com"
+                    }
                 },
                 {
-                    "first_name": "Marie",
-                    "last_name": "Johnson",
-                    "email": "mariejohnson@example.com",
-                    "plan": "basic"
+                    "plan": {
+                      "slug": "basic",
+                      "title": "Basic"
+                    },
+                    "user": {
+                        "username": "mariejohnson",
+                        "slug": "mariejohnson",
+                        "full_name": "Marie Johnson",
+                        "email": "mariejohnson@example.com"
+                    }
                 }
             ],
             "updated": [],
@@ -259,6 +285,9 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
         filed = csv.reader(StringIO(uploaded.read().decode(
             'utf-8', 'ignore')) if uploaded else StringIO())
 
+        at_time = datetime_or_now()
+        status_code = status.HTTP_200_OK
+        resp_serializer = CartItemSerializer()
         for row in filed:
             try:
                 if len(row) == 2:
@@ -278,11 +307,13 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
                           'sync_on': email,
                           'email': email})
                 if serializer.is_valid():
+                    # similar code as `CartItemAPIView.post`
                     cart_item, created = self.insert_item(
-                        request, **serializer.data)
+                        request, at_time=at_time, **serializer.data)
                     if isinstance(cart_item, CartItem):
-                        cart_item = serializer.to_representation(cart_item)
+                        cart_item = resp_serializer.to_representation(cart_item)
                     if created:
+                        status_code = status.HTTP_201_CREATED
                         response['created'].append(cart_item)
                     else:
                         response['updated'].append(cart_item)
@@ -290,7 +321,7 @@ class CartItemUploadAPIView(CartMixin, GenericAPIView):
                     response['failed'].append({'data': serializer.data,
                                                'error': serializer.errors})
 
-        return Response(response)
+        return Response(response, status=status_code)
 
 
 class CouponRedeemAPIView(GenericAPIView):
@@ -446,8 +477,11 @@ class CheckoutAPIView(InvoicablesMixin, BalanceAndCartMixin,
 
         The cart is manipulated through various API endpoints:
 
-        - `Redeems a discount code </docs/api/#createCouponRedeem>`_ applies a coupon code for a potential discount, and
-        - `Adds an item to the request user cart </docs/api/#createCartItem>`_, `Removes an item from the request user cart </docs/api/#destroyCartItem>`_ to update a cart.
+        - `Redeems a discount code </docs/api/#createCouponRedeem>`_ applies \
+a coupon code for a potential discount, and
+        - `Adds an item to the request user cart </docs/api/#createCartItem>`_,\
+ `Removes an item from the request user cart </docs/api/#destroyCartItem>`_\
+ to update a cart.
 
         The API is typically used within an HTML
         `checkout page </docs/themes/#workflow_billing_cart>`_
@@ -487,10 +521,11 @@ of Xia",
         return self.create(request, *args, **kwargs)
 
     def get(self, request, *args, **kwargs):
+        provider = self.invoicables_provider
         resp_data = {
             'processor':
-            self.organization.processor_backend.get_payment_context(
-                self.invoicables_provider,
+            provider.processor_backend.get_payment_context(
+                provider,
                 self.organization.processor_card_key,
                 amount=self.invoicables_lines_price.amount,
                 unit=self.invoicables_lines_price.unit,
