@@ -25,7 +25,6 @@ from __future__ import unicode_literals
 
 from django.http import Http404
 from django.db import transaction
-from django.utils.translation import ugettext_lazy as _
 from rest_framework import status
 from rest_framework.generics import (CreateAPIView, ListAPIView,
     GenericAPIView, RetrieveAPIView)
@@ -34,6 +33,7 @@ from rest_framework.response import Response
 from .serializers import (ChargeSerializer, EmailChargeReceiptSerializer,
     RefundChargeSerializer, ValidationErrorSerializer)
 from .. import signals
+from ..compat import gettext_lazy as _
 from ..docs import OpenAPIResponse, no_body, swagger_auto_schema
 from ..filters import DateRangeFilter, OrderingFilter, SearchFilter
 from ..humanize import as_money
@@ -43,27 +43,8 @@ from ..pagination import TotalPagination
 
 #pylint: disable=no-init
 
-class RetrieveChargeMixin(ChargeMixin):
-    """
-    Mixin for a ``Charge`` object that will first retrieve the state of
-    the ``Charge`` from the processor API.
 
-    This mixin is intended to be used for API requests. Pages should
-    use the parent ChargeMixin and use AJAX calls to retrieve the state
-    of a ``Charge`` in order to deal with latency and service errors
-    from the processor.
-    """
-    model = Charge
-    slug_field = 'processor_key'
-    slug_url_kwarg = 'organization' # See comment in urls.api.subscriber.charges
-
-    def get_object(self, queryset=None):
-        charge = super(RetrieveChargeMixin, self).get_object(queryset)
-        charge.retrieve()
-        return charge
-
-
-class ChargeResourceView(RetrieveChargeMixin, RetrieveAPIView):
+class ChargeResourceView(ChargeMixin, RetrieveAPIView):
     """
     Retrieves a processor charge
 
@@ -226,7 +207,7 @@ class OrganizationChargeListAPIView(SmartChargeListMixin,
     pagination_class = TotalPagination
 
 
-class ChargeRefundAPIView(RetrieveChargeMixin, CreateAPIView):
+class ChargeRefundAPIView(ChargeMixin, CreateAPIView):
     """
     Refunds a processor charge
 
@@ -319,7 +300,7 @@ class ChargeRefundAPIView(RetrieveChargeMixin, CreateAPIView):
         return Response(ChargeSerializer().to_representation(self.object))
 
 
-class EmailChargeReceiptAPIView(RetrieveChargeMixin, GenericAPIView):
+class EmailChargeReceiptAPIView(ChargeMixin, GenericAPIView):
     """
     Re-sends a charge receipt
 
