@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,13 @@
 
 import logging
 
-from django.utils.translation import ugettext_lazy as _
 from rest_framework.generics import GenericAPIView, ListAPIView
 from rest_framework.response import Response
 
 from .serializers import (CartItemSerializer, LifetimeSerializer,
     MetricsSerializer)
 from .. import settings
-from ..compat import reverse, six
+from ..compat import gettext_lazy as _, reverse, six
 from ..filters import DateRangeFilter
 from ..metrics.base import (abs_monthly_balances,
     aggregate_transactions_by_period, month_periods,
@@ -366,7 +365,10 @@ class CouponUsesAPIView(CartItemSmartListMixin, CouponUsesQuerysetMixin,
                         "printable_name": "Xia Doe",
                         "username": "xia"
                     },
-                    "plan": "basic",
+                    "plan": {
+                      "slug": "basic",
+                      "title": "Basic"
+                    },
                     "created_at": "2014-01-01T09:00:00Z"
                 }
             ]
