@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,88 +24,26 @@
 
 import hashlib, os, re
 
+from dateutil.relativedelta import relativedelta
 from django.conf import settings as django_settings
 from django.contrib.auth import get_user_model, logout as auth_logout
 from django.db import transaction, IntegrityError
-from django.utils.encoding import force_text
-from django.utils.translation import ugettext_lazy as _
 from rest_framework import parsers, status
 from rest_framework.generics import (CreateAPIView, ListAPIView,
-    ListCreateAPIView, RetrieveUpdateDestroyAPIView)
+    RetrieveUpdateDestroyAPIView)
 from rest_framework.response import Response
 
-from .serializers import (OrganizationCreateSerializer,
-    OrganizationDetailSerializer, OrganizationWithSubscriptionsSerializer,
-    UploadBlobSerializer)
+from .serializers import (OrganizationDetailSerializer,
+    OrganizationWithSubscriptionsSerializer, UploadBlobSerializer)
 from .. import settings, signals
-from ..compat import urlparse, urlunparse
+from ..compat import force_str, gettext_lazy as _, urlparse, urlunparse
 from ..decorators import _valid_manager
-from ..docs import OpenAPIResponse, swagger_auto_schema
-from ..mixins import (OrganizationMixin, OrganizationSmartListMixin,
-    ProviderMixin, OrganizationDecorateMixin)
+from ..filters import DateRangeFilter, OrderingFilter, SearchFilter
+from ..mixins import (DateRangeContextMixin, OrganizationMixin,
+    OrganizationSmartListMixin, ProviderMixin, OrganizationDecorateMixin)
 from ..models import get_broker
-from ..utils import (full_name_natural_split, get_organization_model,
-    get_role_model, handle_uniq_error, get_picture_storage)
-
-
-#pylint: disable=no-init
-class OrganizationCreateMixin(object):
-
-    user_model = get_user_model()
-
-    def create_organization(self, validated_data):
-        organization_model = get_organization_model()
-        organization = organization_model(
-            slug=validated_data.get('slug', None),
-            full_name=validated_data.get('full_name'),
-            email=validated_data.get('email'),
-            default_timezone=validated_data.get(
-                'default_timezone', settings.TIME_ZONE),
-            phone=validated_data.get('phone', ""),
-            street_address=validated_data.get('street_address', ""),
-            locality=validated_data.get('locality', ""),
-            region=validated_data.get('region', ""),
-            postal_code=validated_data.get('postal_code', ""),
-            country=validated_data.get('country', ""),
-            extra=validated_data.get('extra'))
-        organization.is_personal = (validated_data.get('type') == 'personal')
-        with transaction.atomic():
-            try:
-                if organization.is_personal:
-                    try:
-                        user = self.user_model.objects.get(
-                            username=organization.slug)
-                        if not organization.full_name:
-                            organization.full_name = user.get_full_name()
-                        if not organization.email:
-                            organization.email = user.email
-                        # We are saving the `Organization` after the `User`
-                        # exists in the database so we can retrieve
-                        # the full_name and email from that attached user
-                        # if case they were not provided in the API call.
-                        organization.save()
-                    except self.user_model.DoesNotExist:
-                        #pylint:disable=unused-variable
-                        # We are saving the `Organization` when the `User`
-                        # does not exist so we have a chance to create
-                        # a slug/username.
-                        organization.save()
-                        first_name, mid, last_name = full_name_natural_split(
-                            organization.full_name)
-                        user = self.user_model.objects.create_user(
-                            username=organization.slug,
-                            email=organization.email,
-                            first_name=first_name,
-                            last_name=last_name)
-                    organization.add_manager(user)
-                else:
-                    # When `slug` is not present, `save` would try to create
-                    # one from the `full_name`.
-                    organization.save()
-            except IntegrityError as err:
-                handle_uniq_error(err)
-
-        return organization
+from ..utils import (datetime_or_now, get_organization_model, get_role_model,
+    get_role_serializer, handle_uniq_error, get_picture_storage)
 
 
 class OrganizationQuerysetMixin(OrganizationDecorateMixin):
@@ -162,7 +100,7 @@ class OrganizationDetailAPIView(OrganizationMixin, OrganizationQuerysetMixin,
         }
     """
     lookup_field = 'slug'
-    lookup_url_kwarg = 'organization'
+    lookup_url_kwarg = settings.PROFILE_URL_KWARG
     serializer_class = OrganizationWithSubscriptionsSerializer
     user_model = get_user_model()
 
@@ -323,7 +261,7 @@ class OrganizationPictureAPIView(OrganizationMixin, CreateAPIView):
 
         # tentatively extract file extension.
         parts = os.path.splitext(
-            force_text(uploaded_file.name.replace('\\', '/')))
+            force_str(uploaded_file.name.replace('\\', '/')))
         ext = parts[-1].lower() if len(parts) > 1 else ""
         key_name = "%s%s" % (
             hashlib.sha256(uploaded_file.read()).hexdigest(), ext)
@@ -345,7 +283,7 @@ class OrganizationPictureAPIView(OrganizationMixin, CreateAPIView):
 
 class OrganizationListAPIView(OrganizationSmartListMixin,
                               OrganizationQuerysetMixin,
-                              OrganizationCreateMixin, ListCreateAPIView):
+                              ListAPIView):
     """
     Lists billing profiles
 
@@ -383,77 +321,11 @@ class OrganizationListAPIView(OrganizationSmartListMixin,
     serializer_class = OrganizationDetailSerializer
     user_model = get_user_model()
 
-    def get_serializer_class(self):
-        if self.request.method.lower() == 'post':
-            return OrganizationCreateSerializer
-        return super(OrganizationListAPIView, self).get_serializer_class()
-
-    @swagger_auto_schema(responses={
-      201: OpenAPIResponse("Create successful", OrganizationDetailSerializer)})
-    def post(self, request, *args, **kwargs):
-        """
-        Creates an organization, personal or user profile.
-
-        **Examples**
-
-        .. code-block:: http
-
-            POST /api/profile/ HTTP/1.1
-
-        .. code-block:: json
-
-            {
-              "email": "xia@locahost.localdomain",
-              "full_name": "Xia Lee",
-              "type": "personal"
-            }
-
-        responds
-
-        .. code-block:: json
-
-            {
-              "slug": "xia",
-              "email": "xia@locahost.localdomain",
-              "full_name": "Xia Lee",
-              "printable_name": "Xia Lee",
-              "type": "personal",
-              "credentials": true,
-              "default_timezone": "America/Los_Angeles",
-              "phone": "",
-              "street_address": "",
-              "locality": "",
-              "region": "",
-              "postal_code": "",
-              "country": "US",
-              "is_bulk_buyer": false,
-              "extra": null
-            }
-
-        """
-        return self.create(request, *args, **kwargs)
-
     def paginate_queryset(self, queryset):
         page = super(OrganizationListAPIView, self).paginate_queryset(queryset)
         page = self.decorate_personal(page)
         return page
 
-    def create(self, request, *args, **kwargs):
-        #pylint:disable=unused-argument
-        serializer = OrganizationCreateSerializer(data=request.data)
-        serializer.is_valid(raise_exception=True)
-
-        # creates profile
-        organization = self.create_organization(serializer.validated_data)
-        self.decorate_personal(organization)
-
-        # returns created profile
-        serializer = self.serializer_class(instance=organization,
-            context=self.get_serializer_context())
-        headers = self.get_success_headers(serializer.data)
-        return Response(serializer.data,
-            status=status.HTTP_201_CREATED, headers=headers)
-
 
 class SubscribersQuerysetMixin(OrganizationDecorateMixin, ProviderMixin):
 
@@ -508,3 +380,154 @@ class SubscribersAPIView(OrganizationSmartListMixin,
         }
     """
     serializer_class = OrganizationDetailSerializer
+
+
+class EngagedSubscribersSmartListMixin(object):
+    """
+    reporting entities list which is also searchable and sortable.
+    """
+    date_field = 'user__last_login'
+
+    search_fields = (
+        'first_name',
+        'last_name',
+        'organization_full_name')
+
+    ordering_fields = [('user__last_login', 'created_at'),
+                       ('user__first_name', 'first_name'),
+                       ('user__last_name', 'last_name'),
+                       ('organization__full_name', 'organization_full_name')]
+
+    ordering = ('created_at',)
+
+    filter_backends = (DateRangeFilter, SearchFilter, OrderingFilter)
+
+
+class EngagedSubscribersQuerysetMixin(DateRangeContextMixin,
+                                     SubscribersQuerysetMixin):
+
+    def get_queryset(self):
+        filter_params = {}
+        ends_at = datetime_or_now(self.ends_at)
+        start_at = self.start_at
+        if not start_at:
+            start_at = ends_at - relativedelta(days=7)
+        filter_params.update({
+            'user__last_login__lt': ends_at,
+            'user__last_login__gte': start_at
+        })
+        queryset = get_role_model().objects.filter(
+            organization__in=get_organization_model().objects.filter(
+                is_active=True,
+                subscriptions__plan__organization=self.provider,
+                subscriptions__ends_at__gt=ends_at),
+            **filter_params
+        ).select_related('user', 'organization')
+        return queryset
+
+
+class EngagedSubscribersAPIView(EngagedSubscribersSmartListMixin,
+                               EngagedSubscribersQuerysetMixin,
+                               ListAPIView):
+    """
+    Lists engaged subscribers
+
+    Returns a list of {{PAGE_SIZE}} subscriber profiles which have or
+    had a subscription to a plan provided by {organization}.
+
+    The queryset can be further refined to match a search filter (``q``)
+    and/or a range of dates ([``start_at``, ``ends_at``]),
+    and sorted on specific fields (``o``).
+
+    **Tags**: subscriptions, provider, profilemodel
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/profile/cowork/subscribers/engaged/?o=created_at&ot=desc\
+ HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "count": 1,
+            "next": null,
+            "previous": null,
+            "results": [
+                {
+                "slug": "xia",
+                "full_name": "Xia Lee",
+                "email": "xia@localhost.localdomain",
+                "created_at": "2016-01-14T23:16:55Z",
+                "ends_at": "2017-01-14T23:16:55Z"
+                }
+            ]
+        }
+    """
+    serializer_class = get_role_serializer()
+
+
+class UnengagedSubscribersQuerysetMixin(DateRangeContextMixin,
+                                       SubscribersQuerysetMixin):
+
+    def get_queryset(self):
+        ends_at = datetime_or_now(self.ends_at)
+        kwargs = {'role__user__last_login__lt': ends_at}
+        if self.start_at:
+            kwargs.update({'role__user__last_login__gte': self.start_at})
+        else:
+            kwargs.update({
+                'role__user__last_login__gte': ends_at - relativedelta(
+                    days=settings.INACTIVITY_DAYS)})
+        queryset = get_organization_model().objects.filter(
+            is_active=True,
+            subscribes_to__organization=self.provider,
+            subscriptions__ends_at__gt=ends_at).exclude(**kwargs).distinct()
+        return queryset
+
+
+class UnengagedSubscribersAPIView(OrganizationSmartListMixin,
+                                  UnengagedSubscribersQuerysetMixin,
+                                  ListAPIView):
+    """
+    Lists inactive subscribers
+
+    Returns a list of {{PAGE_SIZE}} subscriber profiles which have or
+    had a subscription to a plan provided by {organization}.
+
+    The queryset can be further refined to match a search filter (``q``)
+    and/or a range of dates ([``start_at``, ``ends_at``]),
+    and sorted on specific fields (``o``).
+
+    **Tags**: subscriptions, provider, profilemodel
+
+    **Examples**
+
+    .. code-block:: http
+
+        GET /api/profile/cowork/subscribers/unengaged/?o=created_at&ot=desc\
+ HTTP/1.1
+
+    responds
+
+    .. code-block:: json
+
+        {
+            "count": 1,
+            "next": null,
+            "previous": null,
+            "results": [
+                {
+                "slug": "xia",
+                "full_name": "Xia Lee",
+                "email": "xia@localhost.localdomain",
+                "created_at": "2016-01-14T23:16:55Z",
+                "ends_at": "2017-01-14T23:16:55Z"
+                }
+            ]
+        }
+    """
+    serializer_class = OrganizationDetailSerializer
