@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -42,7 +42,6 @@ from django.contrib.auth import get_user_model
 from django.contrib.auth.hashers import is_password_usable
 from django.db import transaction
 from django.template.defaultfilters import slugify
-from django.utils.translation import ugettext_lazy as _
 from django_countries.serializer_fields import CountryField
 from rest_framework import serializers
 from rest_framework.exceptions import ValidationError
@@ -50,7 +49,7 @@ from rest_framework.generics import get_object_or_404
 import phonenumbers
 
 from .. import settings
-from ..compat import reverse, six
+from ..compat import gettext_lazy as _, reverse, six
 from ..decorators import _valid_manager
 from ..humanize import as_money
 from ..mixins import as_html_description, product_url
@@ -167,6 +166,14 @@ class NoModelSerializer(serializers.Serializer):
         raise RuntimeError('`update()` should not be called.')
 
 
+class PriceSerializer(NoModelSerializer):
+
+    amount = serializers.IntegerField(
+        help_text=_("amount in unit"))
+    unit = serializers.CharField(
+        help_text=_("Three-letter ISO 4217 code for currency unit (ex: usd)"))
+
+
 class BalanceLineSerializer(serializers.ModelSerializer):
 
     class Meta:
@@ -388,7 +395,7 @@ class OrganizationBalanceSerializer(NoModelSerializer):
 
     balance_amount = serializers.IntegerField(read_only=True,
       help_text=_("balance of all transactions in cents (i.e. 100ths) of unit"))
-    balance_unit = serializers.IntegerField(read_only=True,
+    balance_unit = serializers.CharField(read_only=True,
         help_text=_("three-letter ISO 4217 code for currency unit (ex: usd)"))
 
     class Meta:
@@ -419,19 +426,16 @@ class OrganizationSerializer(serializers.ModelSerializer):
     # when one creates an opt-in subscription.
     # If we don't define ``slug`` here, the serializer validators will raise
     # an exception "Organization already exists in database".
-    slug = serializers.CharField(read_only=True,
+    slug = serializers.SlugField(read_only=True,
         help_text=_("Unique identifier shown in the URL bar"))
     printable_name = serializers.SerializerMethodField(read_only=True,
         help_text=_("Name that can be safely used for display in HTML pages"))
     credentials = serializers.SerializerMethodField(read_only=True,
         help_text=_("True if the account has valid login credentials"))
-    full_name = serializers.CharField(required=False,
-        help_text=_("Full name"))
 
     class Meta:
         model = get_organization_model()
-        fields = ('slug', 'full_name', 'printable_name', 'picture',
-            'type', 'credentials')
+        fields = ('slug', 'printable_name', 'picture', 'type', 'credentials')
         read_only_fields = ('printable_name', 'type', 'credentials')
 
     @staticmethod
@@ -461,10 +465,11 @@ OrganizationSerializer._declared_fields["type"] = \
 
 class OrganizationDetailSerializer(OrganizationSerializer):
 
-    slug = serializers.CharField(required=False,
+    slug = serializers.SlugField(required=False,
         help_text=_("Unique identifier shown in the URL bar"))
+    full_name = serializers.CharField(help_text=_("Full name"))
     default_timezone = serializers.CharField(required=False,
-         help_text=_("Timezone to use when reporting metrics"))
+        help_text=_("Timezone to use when reporting metrics"))
     email = serializers.EmailField(required=False,
         help_text=_("E-mail address"))
     phone = serializers.CharField(required=False, allow_blank=True,
@@ -490,7 +495,7 @@ class OrganizationDetailSerializer(OrganizationSerializer):
 
     class Meta(OrganizationSerializer.Meta):
         fields = OrganizationSerializer.Meta.fields + (
-            'created_at', 'email', 'phone',
+            'full_name', 'created_at', 'email', 'phone',
             'street_address', 'locality', 'region', 'postal_code', 'country',
             'default_timezone', 'is_provider', 'is_bulk_buyer', 'extra',
             'detail')
@@ -557,10 +562,18 @@ class AdvanceDiscountSerializer(serializers.ModelSerializer):
 
 class PlanSerializer(serializers.ModelSerializer):
 
-    slug = serializers.CharField(required=False,
+    slug = serializers.SlugField(required=False,
         help_text=_("Unique identifier shown in the URL bar"))
     title = serializers.CharField(required=False,
         help_text=_("Title for the plan"))
+
+    class Meta:
+        model = Plan
+        fields = ('slug', 'title',)
+
+
+class PlanDetailSerializer(PlanSerializer):
+
     description = serializers.CharField(required=False,
         help_text=_("Free-form text description for the %(object)s") % {
             'object': 'plan'})
@@ -600,14 +613,13 @@ class PlanSerializer(serializers.ModelSerializer):
             " in human-readable form"))
 
     class Meta:
-        model = Plan
-        fields = ('slug', 'title', 'description', 'is_active',
-                  'setup_amount', 'period_amount', 'period_type', 'app_url',
-                  'advance_discounts', 'unit', 'organization', 'extra',
-                  'period_length', 'renewal_type', 'is_not_priced',
-                  'created_at',
-                  'skip_optin_on_grant', 'optin_on_request',
-                  'discounted_period_amount', 'is_cart_item', 'detail')
+        model = PlanSerializer.Meta.model
+        fields = PlanSerializer.Meta.fields + ('description', 'is_active',
+            'setup_amount', 'period_amount', 'period_type', 'app_url',
+            'advance_discounts', 'unit', 'organization', 'extra',
+            'period_length', 'renewal_type', 'is_not_priced',
+            'created_at', 'skip_optin_on_grant', 'optin_on_request',
+            'discounted_period_amount', 'is_cart_item', 'detail')
         read_only_fields = ('app_url',
             'discounted_period_amount', 'is_cart_item', 'detail')
 
@@ -654,15 +666,15 @@ class PlanSerializer(serializers.ModelSerializer):
         return instance
 
 
-class PlanCreateSerializer(PlanSerializer):
+class PlanCreateSerializer(PlanDetailSerializer):
     """
     Serializer to create plans in POST requests
     """
     title = serializers.CharField(required=True,
         help_text=_("Title for the plan"))
 
-    class Meta(PlanSerializer.Meta):
-        fields = PlanSerializer.Meta.fields
+    class Meta(PlanDetailSerializer.Meta):
+        fields = PlanDetailSerializer.Meta.fields
 
 
 class OrganizationInviteSerializer(OrganizationCreateSerializer):
@@ -808,6 +820,12 @@ class TransactionSerializer(serializers.ModelSerializer):
             'dest_account', 'dest_organization', 'dest_amount', 'dest_unit')
 
 
+class ChargeItemSerializer(NoModelSerializer):
+
+    invoiced = TransactionSerializer()
+    refunded = TransactionSerializer(many=True)
+
+
 class CreateOfflineTransactionSerializer(NoModelSerializer):
     """
     Serializer to validate the input that creates an off-line transaction.
@@ -842,14 +860,25 @@ class CartItemSerializer(serializers.ModelSerializer):
     """
     user = get_user_serializer()(
         help_text=_("User the cart belongs to"))
-    plan = PlanRelatedField(
+    plan = PlanSerializer(
         help_text=_("Item in the cart (if plan)"))
+    detail = serializers.SerializerMethodField(read_only=True, required=False,
+        help_text=_("Describes the result of the action"\
+            " in human-readable form"))
 
     class Meta:
         model = CartItem
         fields = ('created_at', 'user', 'plan', 'option', 'full_name',
-            'sync_on', 'email')
-        read_only_fields = ('created_at', 'user')
+            'sync_on', 'email', 'detail')
+        read_only_fields = ('created_at', 'user', 'detail')
+
+    @staticmethod
+    def get_detail(obj):
+        if hasattr(obj, 'detail'):
+            return obj.detail
+        if isinstance(obj, dict):
+            return obj.get('detail')
+        return None
 
 
 class CartItemCreateSerializer(serializers.ModelSerializer):
@@ -959,7 +988,7 @@ class AccessibleCreateSerializer(NoModelSerializer):
     """
     Invite a previously existing or new Organization
     """
-    slug = serializers.CharField(required=False, validators=[
+    slug = serializers.SlugField(required=False, validators=[
         validators.RegexValidator(settings.ACCT_REGEX,
             _("Enter a valid organization slug."), 'invalid')],
         help_text=_("Profile to grant {user} a role onto"))
@@ -1014,10 +1043,11 @@ class RoleCreateSerializer(NoModelSerializer):
     Invite a previously existing or new User
     """
 
-    slug = serializers.CharField(required=False,
+    slug = serializers.SlugField(required=False,
         help_text=_("Username"),
         validators=[validators.RegexValidator(settings.ACCT_REGEX,
             _("Enter a valid username."), 'invalid')])
+    #pylint:disable=protected-access
     email = serializers.EmailField(
         max_length=get_user_model()._meta.get_field('email').max_length,
         required=False,
@@ -1036,6 +1066,7 @@ class RoleCreateSerializer(NoModelSerializer):
         # a validation error if the length is too long but arbitrarly shorten
         # the username.
         user_model = get_user_model()
+        #pylint:disable=protected-access
         max_length = user_model._meta.get_field('username').max_length
         if len(data) > max_length:
             if '@' in data:
