@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,13 +27,13 @@ from collections import OrderedDict
 
 from django.http import Http404
 from django.shortcuts import get_object_or_404
-from django.utils.translation import ugettext_lazy as _
 
 # Implementation Note:
 #
 # saas.settings cannot be imported at this point because this file (extras.py)
 # will be imported before ``django.conf.settings`` is fully initialized.
-from .compat import NoReverseMatch, is_authenticated, reverse, six
+from .compat import (NoReverseMatch, gettext_lazy as _, is_authenticated,
+    reverse, six)
 from .utils import get_organization_model
 
 
@@ -42,7 +42,8 @@ class OrganizationMixinBase(object):
     Returns an ``Organization`` from a URL.
     """
 
-    organization_url_kwarg = 'organization'
+    organization_url_kwarg = 'organization'  # XXX PROFILE_URL_KWARG
+    #organization_url_kwarg = 'profile'       # XXX PROFILE_URL_KWARG
 
     @property
     def organization(self):
@@ -60,8 +61,9 @@ class OrganizationMixinBase(object):
         """
         url_kwargs = {}
         params = kwargs if kwargs else self.kwargs
-        if 'organization' in params:
-            url_kwargs.update({'organization': params['organization']})
+        if self.organization_url_kwarg in params:
+            url_kwargs.update({self.organization_url_kwarg:
+                params[self.organization_url_kwarg]})
         return url_kwargs
 
     def get_context_data(self, **kwargs):
