@@ -1,4 +1,4 @@
-# Copyright (c) 2018, DjaoDjin inc.
+# Copyright (c) 2021, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -22,14 +22,18 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import datetime, re, sys
+import datetime, logging, re, sys
 
 from django.core.management.base import BaseCommand
 from django.db import transaction
 from django.utils.timezone import utc
 
 from ...ledger import export
-from ...models import Organization, Transaction
+from ...models import Transaction
+from ...utils import get_organization_model
+
+LOGGER = logging.getLogger(__name__)
+
 
 class Command(BaseCommand):
     help = 'Import/export transactions in ledger format.'
@@ -129,6 +133,34 @@ def import_transactions(filedesc, create_organizations=False, broker=None,
                 if dest_organization and orig_organization:
                     # Assuming no errors, at this point we have
                     # a full transaction.
+                    #pylint:disable=logging-not-lazy
+                    LOGGER.debug(
+                        "Transaction.objects.using('%(using)s').create("\
+                        "created_at='%(created_at)s',"\
+                        "descr='%(descr)s',"\
+                        "dest_unit='%(dest_unit)s',"\
+                        "dest_amount='%(dest_amount)s',"\
+                        "dest_organization='%(dest_organization)s',"\
+                        "dest_account='%(dest_account)s',"\
+                        "orig_amount='%(orig_amount)s',"\
+                        "orig_unit='%(orig_unit)s',"\
+                        "orig_organization='%(orig_organization)s',"\
+                        "orig_account='%(orig_account)s',"\
+                        "event_id=%(event_id)s)" % {
+                            'using': using,
+                            'created_at': created_at,
+                            'descr': descr,
+                            'dest_unit': dest_unit,
+                            'dest_amount': dest_amount,
+                            'dest_organization': dest_organization,
+                            'dest_account': dest_account,
+                            'orig_amount': dest_amount,
+                            'orig_unit': orig_unit,
+                            'orig_organization': orig_organization,
+                            'orig_account': orig_account,
+                            'event_id':
+                                "'%s'" % reference if reference else "None"
+                        })
                     Transaction.objects.using(using).create(
                         created_at=created_at,
                         descr=descr,
@@ -180,15 +212,16 @@ def parse_line(line, create_organizations=False, broker=None, using='default'):
                     amount = int(float(value) * 100)
                 else:
                     amount = int(value)
+        organization_model = get_organization_model()
         try:
             if create_organizations:
-                organization, _ = Organization.objects.using(
+                organization, _ = organization_model.objects.using(
                     using).get_or_create(slug=organization_slug)
             else:
-                organization = Organization.objects.using(using).get(
+                organization = organization_model.objects.using(using).get(
                     slug=organization_slug)
             return (organization, account, amount, unit)
-        except Organization.DoesNotExist:
+        except organization_model.DoesNotExist:
             sys.stderr.write("error: Cannot find Organization '%s'\n"
                 % organization_slug)
     return (None, None, amount, unit)
