@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,23 +29,22 @@ import logging, re
 
 from django.contrib.auth import REDIRECT_FIELD_NAME, get_user_model
 from django.contrib.auth.hashers import UNUSABLE_PASSWORD_PREFIX
-from django.db.models import Q, F
+from django.db import models, transaction, IntegrityError
 from django.http import Http404
 from django.template.defaultfilters import slugify
-from django.utils.translation import ugettext_lazy as _
-from django.views.generic.detail import SingleObjectMixin
 from rest_framework.generics import get_object_or_404
 
 from . import humanize, settings
 from .cart import cart_insert_item
-from .compat import six, NoReverseMatch, is_authenticated, reverse
+from .compat import (NoReverseMatch, gettext_lazy as _, is_authenticated,
+    reverse, six)
 from .filters import DateRangeFilter, OrderingFilter, SearchFilter
 from .models import (CartItem, Charge, Coupon, Plan, Price,
     RoleDescription, Subscription, Transaction, get_broker, is_broker,
     sum_orig_amount)
 from .utils import (build_absolute_uri, datetime_or_now,
-    get_organization_model, get_role_model, update_context_urls,
-    validate_redirect_url)
+    full_name_natural_split, get_organization_model, get_role_model,
+    handle_uniq_error, update_context_urls, validate_redirect_url)
 from .extras import OrganizationMixinBase
 
 LOGGER = logging.getLogger(__name__)
@@ -113,9 +112,22 @@ class CartMixin(object):
     Mixin to retrieve a user's cart as line items for a checkout workflow.
     """
 
-    @staticmethod
-    def insert_item(request, **kwargs):
-        return cart_insert_item(request, **kwargs)
+    def insert_item(self, request, at_time=None, **kwargs):
+        cart_item, created = cart_insert_item(request, **kwargs)
+        detail = None
+        # insert_item will either return a dict or a CartItem instance
+        # (which cannot be directly serialized).
+        if isinstance(cart_item, CartItem):
+            try:
+                invoicable = self.cart_item_as_invoicable(
+                    cart_item, at_time=at_time)
+                if (not invoicable.get('options') and
+                    len(invoicable.get('lines', [])) == 1):
+                    detail = invoicable['lines'][0].descr
+            except get_organization_model().DoesNotExist:
+                detail = None
+            cart_item.detail = detail
+        return cart_item, created
 
     def get_cart(self):
         """
@@ -139,7 +151,7 @@ class CartMixin(object):
 
     @staticmethod
     def get_cart_options(subscription, created_at=None,
-                               prorate_to=None, cart_item=None):
+                         prorate_to=None, cart_item=None):
         """
         Return a set of lines that must charged Today and a set of choices
         based on current subscriptions that the user might be willing
@@ -158,13 +170,7 @@ class CartMixin(object):
             prorated_amount = plan.prorate_period(created_at, prorate_to)
         full_amount = prorated_amount + plan.period_amount
 
-        coupon = None
-        full_name = None
-        if cart_item:
-            coupon = cart_item.coupon
-            if coupon:
-                if coupon.code.startswith('cpn_'):
-                    full_name = cart_item.full_name
+        coupon = cart_item.coupon if cart_item else None
 
         discount_by_types = {}
         coupon_discount_amount = 0
@@ -195,10 +201,8 @@ class CartMixin(object):
                 created_at=created_at)]
 
         else:
-            nb_periods = subscription.plan.period_length
-            ends_at = subscription.plan.end_of_period(
-                subscription.ends_at, nb_periods)
-
+            ends_at = plan.end_of_period(
+                subscription.ends_at, plan.period_length)
             option_items += [Transaction.objects.new_subscription_order(
                 subscription,
                 amount=amount,
@@ -206,7 +210,7 @@ class CartMixin(object):
                     plan, ends_at, 1,
                     discount_by_types=discount_by_types,
                     coupon=coupon,
-                    full_name=full_name),
+                    cart_item=cart_item),
                 created_at=created_at)]
 
             advance_discounts = plan.advance_discounts.all().order_by('length')
@@ -245,11 +249,100 @@ class CartMixin(object):
                         plan, ends_at, nb_periods,
                         discount_by_types=discount_by_types,
                         coupon=coupon,
-                        full_name=full_name),
+                        cart_item=cart_item),
                     created_at=created_at)]
 
         return option_items
 
+    def cart_item_as_invoicable(self, cart_item, customer=None, at_time=None):
+        created_at = datetime_or_now(at_time)
+        prorate_to_billing = False
+        prorate_to = None
+        if prorate_to_billing:
+            # XXX First we add enough periods to get the next billing date later
+            # than created_at but no more than one period in the future.
+            prorate_to = customer.billing_start
+        subscriber = customer
+        organization_model = get_organization_model()
+        if cart_item.sync_on:
+            try:
+                subscriber = organization_model.objects.get(
+                    models.Q(slug=cart_item.sync_on)
+                    | models.Q(email__iexact=cart_item.sync_on))
+            except organization_model.DoesNotExist:
+                # commit c30c4c58 states "not a groupbuy when sync_on is
+                # a username"
+                user_queryset = get_user_model().objects.filter(
+                    models.Q(username=cart_item.sync_on)
+                    | models.Q(email__iexact=cart_item.sync_on))
+                if not user_queryset.exists():
+                    # XXX Hacky way to determine GroupBuy vs. notify.
+                    subscriber = get_organization_model()(
+                        full_name=cart_item.full_name.strip(),
+                        email=cart_item.sync_on)
+        if not subscriber:
+            # We cannot figure out a subscriber from the cart_item
+            # and not profile was passed as a customer to be billed.
+            raise organization_model.DoesNotExist()
+        try:
+            # If we can extend a current ``Subscription`` we will.
+            # XXX For each (organization, plan) there should not
+            #     be overlapping timeframe [created_at, ends_at[,
+            #     None-the-less, it might be a good idea to catch
+            #     and throw a nice error message in case.
+            subscription = Subscription.objects.get(
+                organization=subscriber, plan=cart_item.plan,
+                ends_at__gt=created_at)
+        except Subscription.DoesNotExist:
+            ends_at = prorate_to if prorate_to else created_at
+            subscription = Subscription.objects.new_instance(
+                subscriber, cart_item.plan, ends_at=ends_at)
+        lines = []
+        options = []
+        if cart_item.use:
+            # We are dealing with an additional use charge instead
+            # of the base subscription.
+            lines += [Transaction.objects.new_use_charge(subscription,
+                cart_item.use, cart_item.option)]
+        else:
+            options = self.get_cart_options(subscription,
+                created_at=created_at, prorate_to=prorate_to,
+                cart_item=cart_item)
+            # option is selected
+            if len(options) == 1:
+                # We only have one option.
+                # It could happen when we use a 100% discount coupon
+                # with advance discounts; in which case the advance
+                # discounts would be cancelled. nothing is totally free.
+                line = options[0]
+                lines += [line]
+                options = []
+            elif (cart_item.option > 0 and
+                (cart_item.option - 1) < len(options)):
+                # The number of periods was already selected so we generate
+                # a line instead.
+                line = options[cart_item.option - 1]
+                lines += [line]
+                options = []
+        # Both ``TransactionManager.new_use_charge``
+        # and ``TransactionManager.new_subscription_order`` will have
+        # created a ``Transaction`` with the ultimate subscriber
+        # as payee. Overriding ``dest_organization`` here
+        # insures in all cases (bulk and direct buying),
+        # the transaction is recorded (in ``execute_order``)
+        # on behalf of the customer on the checkout page.
+        for line in lines:
+            line.dest_organization = customer
+
+        return {
+            'subscription': subscription,
+            'name': cart_item.name,
+            'lines': lines,
+            'options': options,
+            'descr': cart_item.descr,
+        }
+
+
     def as_invoicables(self, user, customer, at_time=None):
         """
         Returns a list of invoicables from the cart of a request user.
@@ -279,123 +372,13 @@ class CartMixin(object):
         """
         #pylint: disable=too-many-locals
         created_at = datetime_or_now(at_time)
-        prorate_to_billing = False
-        prorate_to = None
-        if prorate_to_billing:
-            # XXX First we add enough periods to get the next billing date later
-            # than created_at but no more than one period in the future.
-            prorate_to = customer.billing_start
         invoicables = []
         for cart_item in CartItem.objects.get_cart(user=user):
-            for_descr = ''
-            organization = customer
-            if cart_item.sync_on:
-                full_name = cart_item.full_name.strip()
-                for_descr = ', for %s (%s)' % (full_name, cart_item.sync_on)
-                organization_queryset = get_organization_model().objects.filter(
-                    Q(slug=cart_item.sync_on)
-                    | Q(email__iexact=cart_item.sync_on))
-                if organization_queryset.exists():
-                    organization = organization_queryset.get()
-                else:
-                    user_queryset = get_user_model().objects.filter(
-                        Q(username=cart_item.sync_on)
-                        | Q(email__iexact=cart_item.sync_on))
-                    if not user_queryset.exists():
-                        # XXX Hacky way to determine GroupBuy vs. notify.
-                        organization = get_organization_model()(
-                            full_name=cart_item.full_name,
-                            email=cart_item.sync_on)
-            try:
-                # If we can extend a current ``Subscription`` we will.
-                # XXX For each (organization, plan) there should not
-                #     be overlapping timeframe [created_at, ends_at[,
-                #     None-the-less, it might be a good idea to catch
-                #     and throw a nice error message in case.
-                subscription = Subscription.objects.get(
-                    organization=organization, plan=cart_item.plan,
-                    ends_at__gt=datetime_or_now())
-            except Subscription.DoesNotExist:
-                ends_at = prorate_to
-                if not ends_at:
-                    ends_at = created_at
-                subscription = Subscription.objects.new_instance(
-                    organization, cart_item.plan, ends_at=ends_at)
-            lines = []
-            options = []
-            if cart_item.use:
-                # We are dealing with an additional use charge instead
-                # of the base subscription.
-                lines += [Transaction.objects.new_use_charge(subscription,
-                    cart_item.use, cart_item.option)]
-            else:
-                options = self.get_cart_options(subscription,
-                    created_at=created_at, prorate_to=prorate_to,
-                    cart_item=cart_item)
-                # option is selected
-                if len(options) == 1:
-                    # We only have one option.
-                    # It could happen when we use a 100% discount coupon
-                    # with advance discounts; in which case the advance
-                    # discounts would be cancelled. nothing is totally free.
-                    line = options[0]
-                    lines += [line]
-                    options = []
-                elif (cart_item.option > 0 and
-                    (cart_item.option - 1) < len(options)):
-                    # The number of periods was already selected so we generate
-                    # a line instead.
-                    line = options[cart_item.option - 1]
-                    lines += [line]
-                    options = []
-            # Both ``TransactionManager.new_use_charge``
-            # and ``TransactionManager.new_subscription_order`` will have
-            # created a ``Transaction`` with the ultimate subscriber
-            # as payee. Overriding ``dest_organization`` here
-            # insures in all cases (bulk and direct buying),
-            # the transaction is recorded (in ``execute_order``)
-            # on behalf of the customer on the checkout page.
-            for line in lines:
-                line.dest_organization = customer
-                line.descr += for_descr
-            invoicables += [{
-                'subscription': subscription,
-                'name': cart_item.name,
-                'lines': lines,
-                'options': options,
-                'descr': cart_item.descr,
-            }]
+            invoicables += [self.cart_item_as_invoicable(
+                cart_item, customer, at_time=created_at)]
         return invoicables
 
 
-class ChargeMixin(SingleObjectMixin):
-    """
-    Mixin for a ``Charge`` object.
-    """
-    model = Charge
-    slug_field = 'processor_key'
-    slug_url_kwarg = 'charge'
-
-    def get_context_data(self, **kwargs):
-        context = super(ChargeMixin, self).get_context_data(**kwargs)
-        charge = self.object
-        context.update(get_charge_context(charge))
-        urls = {'charge': {
-            'api_base': reverse('saas_api_charge', args=(charge,)),
-            'api_email_receipt': reverse(
-                'saas_api_email_charge_receipt', args=(charge,)),
-            'api_refund': reverse('saas_api_charge_refund', args=(charge,))}}
-        try:
-            # optional
-            urls['charge'].update({'printable_receipt': reverse(
-                'saas_printable_charge_receipt',
-                args=(charge.customer, charge,))})
-        except NoReverseMatch:
-            pass
-        update_context_urls(context, urls)
-        return context
-
-
 class UserMixin(object):
     """
     Returns an ``User`` from a URL.
@@ -470,6 +453,65 @@ class OrganizationMixin(OrganizationMixinBase, settings.EXTRA_MIXIN):
     pass
 
 
+class OrganizationCreateMixin(object):
+
+    user_model = get_user_model()
+
+    def create_organization(self, validated_data):
+        organization_model = get_organization_model()
+        organization = organization_model(
+            slug=validated_data.get('slug', None),
+            full_name=validated_data.get('full_name'),
+            email=validated_data.get('email'),
+            default_timezone=validated_data.get(
+                'default_timezone', settings.TIME_ZONE),
+            phone=validated_data.get('phone', ""),
+            street_address=validated_data.get('street_address', ""),
+            locality=validated_data.get('locality', ""),
+            region=validated_data.get('region', ""),
+            postal_code=validated_data.get('postal_code', ""),
+            country=validated_data.get('country', ""),
+            extra=validated_data.get('extra'))
+        organization.is_personal = (validated_data.get('type') == 'personal')
+        with transaction.atomic():
+            try:
+                if organization.is_personal:
+                    try:
+                        user = self.user_model.objects.get(
+                            username=organization.slug)
+                        if not organization.full_name:
+                            organization.full_name = user.get_full_name()
+                        if not organization.email:
+                            organization.email = user.email
+                        # We are saving the `Organization` after the `User`
+                        # exists in the database so we can retrieve
+                        # the full_name and email from that attached user
+                        # if case they were not provided in the API call.
+                        organization.save()
+                    except self.user_model.DoesNotExist:
+                        #pylint:disable=unused-variable
+                        # We are saving the `Organization` when the `User`
+                        # does not exist so we have a chance to create
+                        # a slug/username.
+                        organization.save()
+                        first_name, mid, last_name = full_name_natural_split(
+                            organization.full_name)
+                        user = self.user_model.objects.create_user(
+                            username=organization.slug,
+                            email=organization.email,
+                            first_name=first_name,
+                            last_name=last_name)
+                    organization.add_manager(user)
+                else:
+                    # When `slug` is not present, `save` would try to create
+                    # one from the `full_name`.
+                    organization.save()
+            except IntegrityError as err:
+                handle_uniq_error(err)
+
+        return organization
+
+
 class OrganizationDecorateMixin(object):
 
     @staticmethod
@@ -522,7 +564,7 @@ class OrganizationDecorateMixin(object):
         records = [page] if isinstance(page, organization_model) else page
         personal = dict(organization_model.objects.filter(
             pk__in=[profile.pk for profile in records if profile.pk],
-            role__user__username=F('slug')).extra(select={
+            role__user__username=models.F('slug')).extra(select={
             'credentials': ("NOT (password LIKE '" + UNUSABLE_PASSWORD_PREFIX
             + "%%')")}).values_list('pk', 'credentials'))
         for profile in records:
@@ -532,6 +574,29 @@ class OrganizationDecorateMixin(object):
         return page
 
 
+class ChargeMixin(OrganizationMixin):
+    """
+    Mixin for a ``Charge`` object that will first retrieve the state of
+    the ``Charge`` from the processor API.
+
+    This mixin is intended to be used for API requests. Pages should
+    use the parent ChargeMixin and use AJAX calls to retrieve the state
+    of a ``Charge`` in order to deal with latency and service errors
+    from the processor.
+    """
+    model = Charge
+    slug_field = 'processor_key'
+    slug_url_kwarg = 'charge'
+
+    def get_object(self, queryset=None):
+        if not queryset:
+            queryset = self.model.objects.filter(customer=self.organization)
+        kwargs = {self.slug_field: self.kwargs.get(self.slug_url_kwarg)}
+        charge = get_object_or_404(queryset, **kwargs)
+        charge.retrieve()
+        return charge
+
+
 class DateRangeContextMixin(object):
 
     forced_date_range = True
@@ -750,7 +815,7 @@ class MetricsMixin(DateRangeContextMixin, ProviderMixin):
 class SubscriptionMixin(OrganizationDecorateMixin):
 
     model = Subscription
-    subscriber_url_kwarg = 'organization'
+    subscriber_url_kwarg = settings.PROFILE_URL_KWARG
 
     def get_queryset(self):
         kwargs = {}
@@ -1211,15 +1276,15 @@ def _as_html_description(transaction_descr,
     return result
 
 
-def as_html_description(transaction, active_links=True):
+def as_html_description(transaction_model, active_links=True):
     """
     Add hyperlinks into a transaction description.
     """
     return _as_html_description(
-        transaction.descr,
-        transaction.orig_organization,
-        transaction.dest_organization,
-        transaction.dest_account,
+        transaction_model.descr,
+        transaction_model.orig_organization,
+        transaction_model.dest_organization,
+        transaction_model.dest_account,
         active_links=active_links)
 
 
