@@ -343,7 +343,7 @@ var regions = {
 var timezoneMixin = {
     data: function(){
         return {
-            timezone: 'local',
+            timezone: moment.tz.guess(),
         }
     },
     methods: {
@@ -372,6 +372,27 @@ var timezoneMixin = {
             }
             return datetime.clone().subtract(1, 'months').format("MMM'YY");
         },
+        datetimeOrNow: function(dateISOString, offset) {
+            var dateTime = moment(dateISOString);
+            if( dateTime.isValid() ) {
+                return dateISOString;
+            }
+            if( offset === "startOfDay" ) {
+                return moment().startOf('day').toISOString();
+            } else if( offset === "endOfDay" ) {
+                return moment().endOf('day').toISOString();
+            }
+            return moment().toISOString();
+        },
+        // Same as itemListMixin
+        asDateInputField: function(dateISOString) {
+            const dateValue = moment(dateISOString);
+            return dateValue.isValid() ? dateValue.format("YYYY-MM-DD") : null;
+        },
+        asDateISOString: function(dateInputField) {
+            const dateValue = moment(dateInputField, "YYYY-MM-DD");
+            return dateValue.isValid() ? dateValue.toISOString() : null;
+        }
     },
 }
 
@@ -782,6 +803,7 @@ var roleListMixin = {
             showRequested: false,
             profileRequestDone: false,
             inNewProfileFlow: false,
+            candidateId: "",
             unregistered: {
                 slug: '',
                 email: '',
@@ -803,7 +825,7 @@ var roleListMixin = {
             if( jQuery.type(item) === "string" ) {
                 var stringVal = item;
                 item = {slug: "", email: "", full_name: ""};
-                var pattern = /@[a-zA-Z\-]+\.[a-zA-Z\-]{2,3}/;
+                var pattern = /@[a-zA-Z0-9\-]+\.([a-zA-Z\-]{2,3}|localdomain)/;
                 if( pattern.test(stringVal) ) {
                     item['email'] = stringVal;
                 } else {
@@ -830,6 +852,7 @@ var roleListMixin = {
         },
         clearRequestProfile: function() {
             var vm = this;
+            vm.candidateId = "";
             vm.unregistered = {slug: "", email: "", full_name: ""};
             vm.profileRequestDone = false;
             if( vm.$refs.typeahead ) {
@@ -875,7 +898,7 @@ var roleListMixin = {
                 );
             }
         },
-        updateItemSelected: function(item) { // user-typeahead @item-selected="updateItemSelected"
+        updateItemSelected: function(item) {
             var vm = this;
             if( item ) {
                 vm.unregistered = item;
@@ -917,7 +940,13 @@ var roleListMixin = {
         },
         submit: function() {
             var vm = this;
-            this._addRole(vm.unregistered, vm.profileRequestDone);
+            if( vm.unregistered.slug || vm.unregistered.email) {
+                this._addRole(vm.unregistered, vm.profileRequestDone);
+            } else {
+                this._addRole((vm.$refs.account && vm.$refs.account.query) ?
+                    vm.$refs.account.query : vm.candidateId,
+                    vm.profileRequestDone);
+            }
         },
         updateParams: function(){ // internal
             var vm = this;
@@ -966,12 +995,19 @@ var roleListMixin = {
     }
 }
 
-
+/**
+ XXX used in conjunction with ``itemListMixin`` in ``subscriptionListMixin``
+ to compute cut-off dates.
+ */
 var subscriptionDetailMixin = {
+    mixins: [
+        timezoneMixin
+    ],
     data: function(){
         return {
             api_profile_url: this.$urls.organization.api_profile_base,
-            ends_at: moment().endOf("day").format(DATE_FORMAT),
+            ends_at: this.datetimeOrNow(
+                this.$dateRange ? this.$dateRange.ends_at : null, 'endOfDay'),
         }
     },
     methods: {
@@ -993,7 +1029,7 @@ var subscriptionDetailMixin = {
         },
         endsSoon: function(subscription) {
             var vm = this;
-            var cutOff = moment(vm.ends_at, DATE_FORMAT).add(5, 'days');
+            var cutOff = moment(vm.ends_at).add(5, 'days');
             var subEndsAt = moment(subscription.ends_at);
             if( subEndsAt < cutOff ) {
                 return "bg-warning";
@@ -1016,6 +1052,18 @@ var subscriptionDetailMixin = {
             delete item.edit_description;
             this.update(item);
         },
+        toggleEndsAt: function (item, event) {
+            var vm = this;
+            vm.$set(item, '_editEndsAt', !item._editEndsAt);
+            if( item._editEndsAt ) {
+                vm.$nextTick(function(){
+                    vm.$refs['editEndsAt_' + item.code][0].focus();
+                });
+            } else {
+                item.ends_at = vm.asDateISOString(event.target.value);
+                vm.update(item);
+            }
+        },
         update: function(item) {
             var vm = this;
             var url = vm.subscriptionURL(
@@ -1028,12 +1076,13 @@ var subscriptionDetailMixin = {
         },
         acceptRequestURL: function(organization, request_key) {
            var vm = this;
-           return (vm.api_profile_url +
+           return vm._safeUrl(vm.api_profile_url,
                 organization + "/subscribers/accept/" + request_key + "/");
         },
         subscriptionURL: function(organization, plan) {
            var vm = this;
-            return vm.api_profile_url + organization + "/subscriptions/" + plan;
+            return vm._safeUrl(vm.api_profile_url,
+                organization + "/subscriptions/" + plan);
         },
     }
 }
@@ -1143,7 +1192,8 @@ var subscriptionListMixin = {
         },
         subscribersURL: function(provider, plan) {
             var vm = this;
-            return vm.api_profile_url + provider + "/plans/" + plan + "/subscriptions/";
+            return vm._safeUrl(vm.api_profile_url,
+                provider + "/plans/" + plan + "/subscriptions");
         },
     },
     mounted: function(){
@@ -1219,23 +1269,7 @@ var couponDetailMixin = {
             var vm = this;
             vm.reqPut(vm.url + '/' + coupon.code, coupon,
             function(){
-                vm.get();
-                if(cb) cb();
-            });
-        },
-        editPlan: function(item){
-            var vm = this;
-            vm.$set(item, '_editPlan', true);
-            vm.$nextTick(function(){
-                vm.$refs['editPlan_' + item.code][0].focus();
-            });
-        },
-        savePlan: function(item){
-            var vm = this;
-            if(!item._editPlan) return;
-            vm.update(item, function(){
-                vm.$set(item, '_editPlan', false);
-                delete item._editPlan;
+                if(cb) cb(); else vm.get();
             });
         },
         editAttempts: function(item){
@@ -1271,6 +1305,35 @@ var couponDetailMixin = {
                 this.update(this.items.results[idx])
             }
         },
+        toggleEndsAt: function (item, event) {
+            var vm = this;
+            if( item._editEndsAt ) {
+                item.ends_at = vm.asDateISOString(event.target.value);
+                vm.update(item, function() {
+                    vm.$set(item, '_editEndsAt', !item._editEndsAt);
+                });
+            } else {
+                vm.$set(item, '_editEndsAt', !item._editEndsAt);
+                vm.$nextTick(function(){
+                    vm.$refs['editEndsAt_' + item.code][0].focus();
+                });
+            }
+        },
+        editPlan: function(item){
+            var vm = this;
+            vm.$set(item, '_editPlan', true);
+            vm.$nextTick(function(){
+                vm.$refs['editPlan_' + item.code][0].focus();
+            });
+        },
+        savePlan: function(item){
+            var vm = this;
+            if(!item._editPlan) return;
+            vm.update(item, function(){
+                vm.$set(item, '_editPlan', false);
+                delete item._editPlan;
+            });
+        },
     }
 }
 
@@ -1301,7 +1364,7 @@ Vue.component('coupon-list', {
         remove: function(idx){
             var vm = this;
             var code = this.items.results[idx].code;
-            vm.reqDelete(vm.url + '/' + code,
+            vm.reqDelete(vm._safeUrl(vm.url, code),
             function() {
                 vm.get();
             });
@@ -1363,15 +1426,21 @@ Vue.component('coupon-list', {
 });
 
 
+/** Lists users recently registered.
+    XXX only used in saas testsite.
+ */
 Vue.component('user-list', {
     mixins: [
-        itemListMixin
+        itemListMixin,
+        timezoneMixin
     ],
     data: function() {
         return {
             url: this.$urls.provider.api_accounts,
             params: {
-                start_at: moment().startOf('day'),
+                start_at: this.datetimeOrNow(
+                    this.$dateRange ? this.$dateRange.start_at : null,
+                    'startOfDay'),
                 o: '-created_at'
             }
         }
@@ -1440,6 +1509,8 @@ Vue.component('role-user-list', {
 });
 
 
+/** XXX We need an ends_at and timezone to compute metrics accurately.
+ */
 Vue.component('metrics-charts', {
     mixins: [
         httpRequestMixin,
@@ -1450,24 +1521,18 @@ Vue.component('metrics-charts', {
             tables: this.$tables,
             activeTab: 0,
             params: {
-                ends_at: moment(),
+                ends_at: this.datetimeOrNow(
+                    this.$dateRange ? this.$dateRange.ends_at : null),
             },
         }
-        if( this.$dateRange && this.$dateRange.ends_at ) {
-            var ends_at = moment(this.$dateRange.ends_at);
-            if(ends_at.isValid()){
-                data.params.ends_at = ends_at;
-            }
-        }
-        data.params.ends_at = data.params.ends_at.format(DATE_FORMAT);
         return data;
     },
     methods: {
         fetchTableData: function(table, cb){
             var vm = this;
-            var params = {"ends_at": moment(vm.params.ends_at, DATE_FORMAT).toISOString()};
+            var params = {ends_at: vm.params.ends_at};
             if( vm.timezone !== 'utc' ) {
-                params["timezone"] = moment.tz.guess();
+                params["timezone"] = vm.timezone;
             }
             vm.reqGet(table.location, params, function(resp){
                 var unit = resp.unit;
@@ -1570,11 +1635,27 @@ Vue.component('metrics-charts', {
                 res = data[0].values;
             }
             return res;
+        },
+        // Same as in `itemListMixin`.
+        _ends_at: {
+            get: function() {
+                // form field input="date" will expect ends_at as a String
+                // but will literally cut the hour part regardless of timezone.
+                // We don't want an empty list as a result.
+                // If we use moment `endOfDay` we get 23:59:59 so we
+                // add a full day instead.
+                const dateValue = moment(this.params.ends_at).add(1,'days');
+                return dateValue.isValid() ? dateValue.format("YYYY-MM-DD") : null;
+            },
+            set: function(newVal) {
+                this.$set(this.params, 'ends_at', this.asDateISOString(newVal));
+                this.get();
+            }
         }
     },
     mounted: function(){
         var vm = this;
-        vm.prepareCurrentTabData();
+        vm.get();
         if( false ) {
             // XXX donot pretetch other tabs to match angularjs code
             //     and pass tests.
@@ -1749,36 +1830,36 @@ Vue.component('plan-list', {
     }
 });
 
-
+/** XXX default value to import new transactions.
+ */
 Vue.component('import-transaction', {
     mixins: [
-        httpRequestMixin
+        httpRequestMixin,
+        timezoneMixin,
     ],
     data: function() {
         return {
             url: this.$urls.organization.api_import,
             typeaheadUrl: this.$urls.provider.api_subscribers_active,
             itemSelected: '',
-            searching: false,
-            entry: {
-                subscription: null,
-                created_at: moment().format("YYYY-MM-DD"),
-                amount: 0,
-                descr: '',
-            },
-            params: {
-                timezone: 'local'
-            }
+            entry: this.defaultNewItem(),
         }
     },
     methods: {
+        defaultNewItem: function() {
+            return {
+                subscription: null,
+                created_at: this.datetimeOrNow(),
+                amount: 0,
+                descr: '',
+            }
+        },
         addPayment: function(){
             var vm = this;
             if( vm.itemSelected ) {
                 vm.entry.subscription = (vm.itemSelected.organization.slug
                     + ':' + vm.itemSelected.plan.slug);
             }
-            vm.entry.created_at = moment(vm.entry.created_at).toISOString();
             vm.reqPost(vm.url, vm.entry,
             function(resp) {
                 vm.clearNewPayment();
@@ -1790,30 +1871,11 @@ Vue.component('import-transaction', {
         clearNewPayment: function() {
             var vm = this;
             vm.itemSelected = '';
-            vm.entry = {
-                subscription: null,
-                created_at: moment().format("YYYY-MM-DD"),
-                amount: 0,
-                descr: '',
-            };
+            vm.entry = vm.defaultNewItem();
         },
         get: function() {
             // We want to keep a single template for `date_input_field`.
         },
-        getSubscriptions: function(query, done) {
-            var vm = this;
-            vm.searching = true;
-            vm.reqGet(vm.typeaheadUrl, {q: query}, function(res){
-                vm.searching = false;
-                // current typeahead implementation does not
-                // support dynamic keys that's why we are
-                // creating them here
-                res.results.forEach(function(e){
-                    e.itemKey = e.organization.slug + ':' + e.plan.slug
-                });
-                done(res.results)
-            });
-        },
         updateItemSelected: function(item) {
             var vm = this;
             if( item ) {
@@ -2096,7 +2158,7 @@ Vue.component('balance-list', {
         },
         remove: function(id){
             var vm = this;
-            vm.reqDelete(vm.balanceLineUrl + '/' + id, function() {
+            vm.reqDelete(vm._safeUrl(vm.balanceLineUrl, id), function() {
                 vm.get()
             });
         },
@@ -2571,6 +2633,57 @@ Vue.component('plan-update', {
     },
 });
 
+// Widgets for activity page
+// -------------------------
+Vue.component('engaged-subscribers', {
+    mixins: [
+        itemListMixin
+    ],
+    data: function(){
+        return {
+            url: this.$urls.api_engaged_subscribers,
+            messagesElement: '#engaged-subscribers-content',
+            scrollToTopOnMessages: false
+        }
+    },
+    methods: {
+    },
+    mounted: function(){
+        this.params.ends_at = null;
+        this.get();
+    },
+});
+
+
+Vue.component('unengaged-subscribers', {
+    mixins: [
+        itemListMixin
+    ],
+    data: function(){
+        return {
+            url: this.$urls.api_unengaged_subscribers,
+            messagesElement: '#unengaged-subscribers-content',
+            scrollToTopOnMessages: false
+        }
+    },
+    methods: {
+        showHideRoles: function(entry) {
+            var vm = this;
+            if( !entry.roles ) {
+                vm.reqGet(vm._safeUrl(vm.$urls.organization.api_profile_base,
+                    entry.slug) + '/roles',
+                function success(resp) {
+                    // Vue.set(entry, 'roles', [{printable_name: "Alice"}]);
+                    Vue.set(entry, 'roles', resp.results);
+                });
+            }
+        }
+    },
+    mounted: function(){
+        this.get()
+    },
+});
+
 
 // Widgets for dashboard
 // ---------------------
@@ -2589,13 +2702,16 @@ Vue.component('search-profile', {
 
 Vue.component('today-sales', {
     mixins: [
-        itemListMixin
+        itemListMixin,
+        timezoneMixin
     ],
     data: function() {
         return {
             url: this.$urls.provider.api_receivables,
             params: {
-                start_at: moment().startOf('day'),
+                start_at: this.datetimeOrNow(
+                    this.$dateRange ? this.$dateRange.start_at : null,
+                    'startOfDay'),
                 o: '-created_at',
             }
         }
