@@ -1,4 +1,4 @@
-# Copyright (c) 2021, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,29 +26,39 @@
 API URLs for a provider subcribers.
 """
 
-from django.conf.urls import url
-
-from ....api.organizations import SubscribersAPIView
+from .... import settings
+from ....api.organizations import (EngagedSubscribersAPIView,
+    UnengagedSubscribersAPIView, SubscribersAPIView)
 from ....api.subscriptions import (ProvidedSubscriptionsAPIView,
     PlanSubscriptionDetailAPIView)
-from ....settings import ACCT_REGEX, VERIFICATION_KEY_RE
 from ....api.subscriptions import SubscriptionRequestAcceptAPIView
+from ....compat import path, re_path
 
 
 urlpatterns = [
-    url(r'^profile/(?P<organization>%s)/subscribers/accept/'\
-        '(?P<request_key>%s)/' % (ACCT_REGEX, VERIFICATION_KEY_RE),
+    re_path(r'profile/(?P<%s>%s)/subscribers/accept/(?P<request_key>%s)' % (
+        settings.PROFILE_URL_KWARG, settings.SLUG_RE,
+        settings.VERIFICATION_KEY_RE),
         SubscriptionRequestAcceptAPIView.as_view(),
         name='saas_api_subscription_grant_accept'),
-    url(r'^profile/(?P<organization>%s)/subscribers/?' % ACCT_REGEX,
+    path('profile/<slug:%s>/subscribers/engaged' %
+        settings.PROFILE_URL_KWARG,
+        EngagedSubscribersAPIView.as_view(),
+        name='saas_api_engaged_subscribers'),
+    path('profile/<slug:%s>/subscribers/unengaged' %
+        settings.PROFILE_URL_KWARG,
+        UnengagedSubscribersAPIView.as_view(),
+        name='saas_api_unengaged_subscribers'),
+    path('profile/<slug:%s>/subscribers' %
+        settings.PROFILE_URL_KWARG,
         SubscribersAPIView.as_view(), name='saas_api_subscribers'),
-    url(r'^profile/(?P<organization>%s)/plans/(?P<plan>%s)/subscriptions/'\
-    '(?P<subscriber>%s)/'
-        % (ACCT_REGEX, ACCT_REGEX, ACCT_REGEX),
+    path(
+    'profile/<slug:%s>/plans/<slug:plan>/subscriptions/<slug:subscriber>' %
+        settings.PROFILE_URL_KWARG,
         PlanSubscriptionDetailAPIView.as_view(),
         name='saas_api_plan_subscription'),
-    url(r'^profile/(?P<organization>%s)/plans/(?P<plan>%s)/subscriptions/'
-        % (ACCT_REGEX, ACCT_REGEX),
+    path('profile/<slug:%s>/plans/<slug:plan>/subscriptions' %
+        settings.PROFILE_URL_KWARG,
         ProvidedSubscriptionsAPIView.as_view(),
         name='saas_api_plan_subscriptions'),
 ]
