@@ -32,13 +32,12 @@ from django.http.request import split_domain_port, validate_host
 from django.template.defaultfilters import slugify
 from django.utils.dateparse import parse_datetime
 from django.utils.timezone import utc, get_current_timezone
-from django.utils.translation import ugettext_lazy as _
 from rest_framework.exceptions import ValidationError
 from rest_framework.settings import api_settings
 from pytz import timezone, UnknownTimeZoneError
 from pytz.tzinfo import DstTzInfo
 
-from .compat import get_model_class, import_string, six
+from .compat import get_model_class, gettext_lazy as _, import_string, six
 
 
 class SlugTitleMixin(object):
@@ -90,6 +89,13 @@ def convert_dates_to_utc(dates):
     return [date.astimezone(utc) for date in dates]
 
 
+def as_timestamp(dtime_at=None):
+    if not dtime_at:
+        dtime_at = datetime_or_now()
+    return int((
+        dtime_at - datetime.datetime(1970, 1, 1, tzinfo=utc)).total_seconds())
+
+
 def datetime_or_now(dtime_at=None):
     if isinstance(dtime_at, six.string_types):
         dtime_at = parse_datetime(dtime_at)
