@@ -45,12 +45,12 @@ from django.db import transaction
 from django.db.models import Q
 from django.contrib import messages
 from django.shortcuts import get_object_or_404
-from django.utils.translation import ugettext_lazy as _
-from django.views.generic import (DetailView, FormView, ListView, TemplateView,
+from django.views.generic import (DetailView, FormView, TemplateView,
     UpdateView)
 from django.utils.http import urlencode
 
 from .. import settings, humanize
+from ..compat import NoReverseMatch, gettext_lazy as _
 from ..cart import session_cart_to_database
 from ..compat import is_authenticated, reverse, six
 from ..backends import ProcessorError, ProcessorConnectionError
@@ -59,7 +59,7 @@ from ..forms import (BankForm, CartPeriodsForm, CreditCardForm,
     ImportTransactionForm, RedeemCouponForm, VTChargeForm, WithdrawForm)
 from ..mixins import (BalanceDueMixin, BalanceAndCartMixin, ChargeMixin,
     DateRangeContextMixin, InvoicablesMixin, OrganizationMixin,
-    ProviderMixin, product_url)
+    ProviderMixin, get_charge_context, product_url)
 from ..models import (CartItem, Charge, Coupon,
     Plan, Price, Subscription, Transaction, UseCharge, get_broker)
 from ..utils import (get_organization_model, update_context_urls,
@@ -320,9 +320,10 @@ class CheckoutFormMixin(CardFormMixin):
         try:
             # computed in `InvoicablesMixin.get_context_data`
             lines_price = context.get('lines_price')
+            provider = self.invoicables_provider
             context.update(
-                self.organization.processor_backend.get_payment_context(
-                    self.invoicables_provider,
+                provider.processor_backend.get_payment_context(
+                    provider,
                     self.organization.processor_card_key,
                     amount=lines_price.amount, unit=lines_price.unit,
                     broker_fee_amount=self.invoicables_broker_fee_amount,
@@ -390,9 +391,10 @@ class CardUpdateView(CardFormMixin, FormView):
         context = super(CardUpdateView, self).get_context_data(**kwargs)
         try:
             # computed in `InvoicablesMixin.get_context_data`
+            broker = get_broker()
             context.update(
-                self.organization.processor_backend.get_payment_context(
-                    get_broker(),
+                broker.processor_backend.get_payment_context(
+                    broker,
                     self.organization.processor_card_key,
                     subscriber_email=self.organization.email,
                     subscriber_slug=self.organization.slug))
@@ -871,7 +873,7 @@ class ChargeListView(ProviderMixin, TemplateView):
         return context
 
 
-class ChargeReceiptView(ChargeMixin, ProviderMixin, DetailView):
+class ChargeReceiptView(ProviderMixin, ChargeMixin, DetailView):
                         # ``ProviderMixin`` to include menubar urls.
     """
     Display a receipt for a ``Charge``.
@@ -898,6 +900,25 @@ djaodjin-saas/tree/master/saas/templates/saas/billing/receipt.html>`__).
 
     def get_context_data(self, **kwargs):
         context = super(ChargeReceiptView, self).get_context_data(**kwargs)
+        charge = self.object
+        context.update(get_charge_context(charge))
+        urls = {'charge': {
+            'api_base': reverse('saas_api_charge', args=(
+                charge.customer, charge,)),
+            'api_email_receipt': reverse(
+                'saas_api_email_charge_receipt', args=(
+                charge.customer, charge,)),
+            'api_refund': reverse('saas_api_charge_refund', args=(
+                charge.customer, charge,))}}
+        try:
+            # optional
+            urls['charge'].update({'printable_receipt': reverse(
+                'saas_printable_charge_receipt',
+                args=(charge.customer, charge,))})
+        except NoReverseMatch:
+            pass
+        update_context_urls(context, urls)
+
         for rank, line in enumerate(context['charge_items']):
             event = line.invoiced.get_event()
             setattr(line, 'rank', rank)
@@ -906,7 +927,7 @@ djaodjin-saas/tree/master/saas/templates/saas/billing/receipt.html>`__).
         return context
 
 
-class CouponListView(ProviderMixin, ListView):
+class CouponListView(DateRangeContextMixin, ProviderMixin, TemplateView):
     """
     View to manage discounts (i.e. ``Coupon``)
 
@@ -930,17 +951,12 @@ coupons.html>`__).
 
     def get_context_data(self, **kwargs):
         context = super(CouponListView, self).get_context_data(**kwargs)
-        urls_provider = {
-            'download_coupons': reverse(
-                'saas_metrics_coupons_download', args=(self.provider,))
-        }
-        if 'urls' in context:
-            if 'provider' in context['urls']:
-                context['urls']['provider'].update(urls_provider)
-            else:
-                context['urls'].update({'provider': urls_provider})
-        else:
-            context.update({'urls': {'provider': urls_provider}})
+        update_context_urls(context, {
+            'provider': {
+                'download_coupons': reverse(
+                    'saas_metrics_coupons_download', args=(self.provider,))
+            }
+        })
         return context
 
 
