@@ -83,14 +83,13 @@ class RoleDetailView(RoleDescriptionMixin, TemplateView):
         context = super(RoleDetailView, self).get_context_data(**kwargs)
         role = self.kwargs.get('role', None)
         context.update({'role_descr': self.role_description})
-        urls = {
+        update_context_urls(context, {
             'api_candidates': reverse('saas_api_search_users'),
             'organization': {
                 'api_roles': reverse(
                     'saas_api_roles_by_descr', args=(
                         self.organization, role)),
-        }}
-        update_context_urls(context, urls)
+        }})
         return context
 
 
@@ -104,13 +103,12 @@ class RoleListView(OrganizationMixin, TemplateView):
 
     def get_context_data(self, **kwargs):
         context = super(RoleListView, self).get_context_data(**kwargs)
-        urls = {'organization': {
+        update_context_urls(context, {'organization': {
             'api_roles': reverse(
                 'saas_api_roles', args=(self.organization,)),
             'api_role_descriptions': reverse(
                 'saas_api_role_description_list', args=(self.organization,)),
-        }}
-        update_context_urls(context, urls)
+        }})
         return context
 
 
@@ -155,8 +153,13 @@ djaodjin-saas/tree/master/saas/templates/saas/profile/subscribers.html>`__).
               'saas_subscriber_pipeline_download_churned', args=(provider,))
             }}]
         context.update({'tabs': tabs})
+        update_context_urls(context, {
+            'subscribers_activity': reverse('saas_subscribers_activity',
+            args=(provider,))
+        })
         if provider.is_broker:
-            context.update({'registered': {'urls': {'download': reverse(
+            update_context_urls(context, {
+                'registered': {'urls': {'download': reverse(
                 'saas_subscriber_pipeline_download_registered')}}})
         return context
 
@@ -178,8 +181,9 @@ subscribers.html>`__).
     def get_context_data(self, **kwargs):
         context = super(PlanSubscribersListView, self).get_context_data(
             **kwargs)
-        context['urls']['provider']['api_plan_subscribers'] = reverse(
-            'saas_api_plan_subscriptions', args=(self.provider, self.plan))
+        update_context_urls(context, {
+            'provider': {'api_plan_subscribers': reverse(
+            'saas_api_plan_subscriptions', args=(self.provider, self.plan))}})
         return context
 
 
@@ -245,6 +249,7 @@ class OrganizationCreateView(RedirectFormMixin, CreateView):
 
     model = get_organization_model()
     organization_model = get_organization_model()
+    organization_url_kwarg = settings.PROFILE_URL_KWARG
     form_class = OrganizationCreateForm
     pattern_name = 'saas_organization_cart'
     template_name = "saas/profile/new.html"
@@ -290,7 +295,7 @@ class OrganizationCreateView(RedirectFormMixin, CreateView):
         return redirect_path
 
     def get_success_url(self):
-        self.kwargs.update({'organization': self.object})
+        self.kwargs.update({self.organization_url_kwarg: self.object})
         success_url = self.get_redirect_url(*self.args, **self.kwargs)
         return str(success_url)
 
@@ -327,23 +332,26 @@ class DashboardView(OrganizationMixin, DetailView):
     """
 
     model = get_organization_model()
-    slug_url_kwarg = 'organization'
+    slug_url_kwarg = settings.PROFILE_URL_KWARG
     template_name = 'saas/metrics/dashboard.html'
 
     def get_context_data(self, **kwargs):
         context = super(DashboardView, self).get_context_data(**kwargs)
+        update_context_urls(context, {
+            'accounts_base': reverse('saas_profile'),
+            'provider': {
+                'bank': reverse('saas_update_bank', args=(self.organization,))
+            }
+        })
         if self.organization.is_broker:
-            urls = {
-                'accounts_base': reverse('saas_profile'),
+            update_context_urls(context, {
                 'provider': {
-                    'api_accounts': reverse('saas_api_search_accounts')}}
+                    'api_accounts': reverse('saas_api_search_accounts')}})
         else:
-            urls = {
-                'accounts_base': reverse('saas_profile'),
+            update_context_urls(context, {
                 'provider': {
-                    'api_accounts': reverse(
-                        'saas_api_subscribers', args=(self.organization,))}}
-        update_context_urls(context, urls)
+                    'api_accounts': reverse('saas_api_subscribers',
+                        args=(self.organization,))}})
         return context
 
     def get_object(self, queryset=None):
@@ -369,7 +377,7 @@ class OrganizationProfileView(OrganizationMixin, UpdateView):
     model = get_organization_model()
     form_class = OrganizationForm
     slug_field = 'slug'
-    slug_url_kwarg = 'organization'
+    slug_url_kwarg = settings.PROFILE_URL_KWARG
     template_name = "saas/profile/index.html"
 
     def update_attached_user(self, form):
